# PrintOut.tcl: generates tcl file for model with all values inserted
# Heinz Kuo
# schwmmrl@uclink4.berkeley.edu
# 08/08/2002
#
# Reads the GenerateBridge.tcl script and returns a list containing the 
# input variables required for GenerateBridge and the corresponding 
# variables extracted from a file containing a table of values
#
# input 1. procfile - name of tcl model generation script
#		2. procnum - tag of the analysis parameter number
#		3. paramline - list of arbitrary length variables corresponding to
#		   the arguments passed into procfile
#
# output - deparameterized OpenSees file named BridgeModelxxxx.tcl
#		   where xxxx corresponds to procnum

proc PrintOut {procfile procnum paramline} {

source "parsely/isfrontier.tcl"
source "parsely/retprocvar.tcl"

set contvar 1

set file [open $procfile "r"]

while {$contvar} {
   gets $file line
   
   if {[isfrontier $line "proc"]} {
      set contvar 0
      set varLIST [retprocvar $line ""]
      set varLENGTH [llength $varLIST]
   }
}

set valLIST [split $paramline]
set valLENGTH [llength $valLIST]

if {[expr $valLENGTH != $varLENGTH]} {
   puts "PrintOut: arguments passed do not agree with $procfile"
   exit
} else {
   foreach thevalue $valLIST {
      lappend varLIST $thevalue
   } 
}

source parsely.tcl
parsely $procfile $procnum $varLIST

}
