# ISFRONTIER: returns whether the input string is a line beginning a for, while,
#             if, switch, ifelse, else, section fiberSec, and plain Patterncase loop.
#             1 if it is, 0 otherwise
# 
# loopt:   1) set       2) proc     3) break    4) pound    5) for
#          6) while     7) while    8) if       9) elseif  10) else
#                      11) switch  12) section 13) pattern
#         
#
# Heinz Kuo, 08/06/02
# schwmmrl@uclink4.berkeley.edu
#
# routine to be used in PARSELY.TCL 
# example:   isfrontier string "for"
# 
proc isfrontier {lineend loopt} {
   source "parsely/countelements.tcl"

   # Make input all lower case
   set loopt [string tolower $loopt]
   set lineend [string tolower $lineend]

   # Initialize Return Variables
   set isGood 1

   switch $loopt {
      "set" {
              set firstappearance [string first "set" $lineend]

              for {set k 0} {$k < $firstappearance} {incr k} {
                 set carrot [string index $lineend $k]
                 if {![string match " " $carrot] && \
                     ![string match "" $carrot]} {set isGood 0;  break}
              }

              set numberofelements [lindex [countelements $lineend] 0]

              if {$numberofelements != 3} {set isGood 0}
            }
      "proc" {
              set firstappearance [string first "proc" $lineend]

              if {$firstappearance >= 0} {
                 set temporary [split $lineend "\{\}"]
                 if {[llength $temporary] != 4} {set isGood 0}

                 for {set k 0} {$k < $firstappearance} {incr k} {
                    set carrot [string index $lineend $k]
                    if {![string match " " $carrot]} {set isGood 0;  break}
                 }
              } else {set isGood 0}
             }
      "pound" {
                set firstappearance [string first "\#" $lineend]

                if {$firstappearance >= 0} {
                   for {set k 0} {$k < $firstappearance} {incr k} {
                      set carrot [string index $lineend $k]
                      if {![string match " " $carrot]} {set isGood 0;  break}
                   }
                } else {set isGood 0}
              }
      "for" {
              set firstappearance [string first "for" $lineend]

              if {$firstappearance >= 0} {
                 set temporary [split $lineend "\{\}"]
                 if {[llength $temporary] != 8} {set isGood 0}

                 for {set k 0} {$k < $firstappearance} {incr k} {
                    set carrot [string index $lineend $k]
                    if {![string match " " $carrot]} {set isGood 0;  break}
                 }
              } else {set isGood 0}
            }
      "while" {
                set firstappearance [string first "while" $lineend]

                if {$firstappearance >= 0} {
                   set temporary [split $lineend "\{\}"]
                   if {[llength $temporary] != 4} {set isGood 0}

                   for {set k 0} {$k < $firstappearance} {incr k} {
                      set carrot [string index $lineend $k]
                      if {![string match " " $carrot]} {set isGood 0;  break}
                   }
                } else {set isGood 0}
              }
      "if" {
             set firstappearance [string first "if" $lineend]
             set notthisthingy [string first "elseif" $lineend]

             if {[expr $firstappearance >= 0] && [expr $notthisthingy == -1]} {
                set temporary [split $lineend "\{\}"]
                if {[llength $temporary] != 4} {set isGood 0}

                for {set k 0} {$k < $firstappearance} {incr k} {
                   set carrot [string index $lineend $k]
                   if {![string match " " $carrot]} {set isGood 0;  break}
                }
             } else {set isGood 0}
           }
      "elseif" {
                 set firstappearance [string first "elseif" $lineend]

                 if {$firstappearance >= 0} {
                    set temporary [split $lineend "\{\}"]
                    if {[llength $temporary] != 5} {set isGood 0}

                    set countcurlies 0
                    for {set k 0} {$k < $firstappearance} {incr k} {
                       set carrot [string index $lineend $k]
                       if {![string match " " $carrot]} {
                          if {$countcurlies > 1} {
                             set isGood 0
                             break
                          } else {
                             set countcurlies [expr $countcurlies+1]
                          }
                       }
                    }
                 } else {set isGood 0}
               }
      "else" {
               set firstappearance [string first "else" $lineend]
               set notthisthingy [string first "elseif" $lineend]

               if {[expr $firstappearance >= 0] && [expr $notthisthingy ==-1]} {
                  set temporary [split $lineend "\{\}"]
                  if {[llength $temporary] != 3} {set isGood 0}

                  set countcurlies 0
                  for {set k 0} {$k < $firstappearance} {incr k} {
                     set carrot [string index $lineend $k]
                     if {![string match " " $carrot]} {
                        if {$countcurlies > 1} {
                           set isGood 0
                           break
                        } else {
                           set countcurlies [expr $countcurlies+1]
                        }
                     }
                  }
               } else {set isGood 0}
             }
      "switch" {
                 set firstappearance [string first "switch" $lineend]

                 if {$firstappearance >= 0} {
                    set temporary [split $lineend "\{\}"]
                    if {[llength $temporary] != 2} {set isGood 0}

                    for {set k 0} {$k < $firstappearance} {incr k} {
                       set carrot [string index $lineend $k]
                       if {![string match " " $carrot]} {set isGood 0;  break}
                    }
                 } else {set isGood 0}
               }
      "section" {
                  set sectappearance [string first "section" $lineend]
                  set fibrappearance [string first "fibersec" $lineend]          

                  if {$sectappearance >= 0} {
                    set temporary [split $lineend "\{"]
                    if {[llength $temporary] != 2} {set isGood 0}

                    for {set k 0} {$k < $sectappearance} {incr k} {
                       set carrot [string index $lineend $k]
                       if {![string match " " $carrot]} {set isGood 0;  break}
                    }
                  } else {set isGood 0}
                }
      "pattern" { 
                  set patappearance [string first "pattern" $lineend]
                  set plainappearance [string first "plain" $lineend]          

                  if {$patappearance >= 0} {
                    set temporary [split $lineend "\{"]
                    if {[llength $temporary] != 2} {set isGood 0}

                    for {set k 0} {$k < $patappearance} {incr k} {
                       set carrot [string index $lineend $k]
                       if {![string match " " $carrot]} {set isGood 0;  break}
                    }
                  } else {set isGood 0}
                }
      "break" {
                set firstappearance [string first "break" $lineend]

                if {$firstappearance >= 0} {
                   for {set k 0} {$k < $firstappearance} {incr k} {
                      set carrot [string index $lineend $k]
                      if {![string match " " $carrot] || \
                          ![string match "" $carrot]} {set isGood 0;  break}
                   }

                   if {$isGood == 1} {
                      set firsttemp [expr $firstappearance + 5]
                      set secondtemp [expr [string length $lineend] - 1]

                      for {set k $firsttemp} {$k < $secondtemp} {incr k} {
                         set carrot [string index $lineend $k]
                         if {![string match " " $carrot] || \
                             ![string match ";" $carrot] || \
                             ![string match ""  $carrot] } {set isGood 0;  break}
                      }

                      unset firsttemp;  unset secondtemp
                   }
                } else {set isGood 0}
              }
      "return" {  
                  set firstappearance [string first "return" $lineend]
                  if {$firstappearance >= 0} {
                     for {set k 0} {$k < $firstappearance} {incr k} {
                        set carrot [string index $lineend $k]
                        if {![string match " " $carrot] || \
                            ![string match "" $carrot]} {set isGood 0;  break}
                     }
                  } else {set isGood 0}
               }
      "default" {
                  puts "error:  incorrect case /ISFRONTIER.TCL"
                  set isGood ""
                }
   }

   return $isGood
}
