# ISSOURCE: returns whether the input string is sourcing a file or not and the file's name
#         option 0 - returns whether it is sourcing or not.  1 if it is, 0 if not
#         option 1 - returns the first name (extensionless name) of the file and without directories
#         option 2 - returns the directory associated with the file...  ie., the steps needed to get 
#                    from the working directory to the directory that houses the file
#         option 3 - reutrns the full name (including extension name) of the file
# 
# Heinz Kuo, 08/06/02
# schwmmrl@uclink4.berkeley.edu
#
# routine to be used in PARSELY.TCL 
# example:   issource string option
proc issource {lineend reopt} {
   set issourcer 1
   set final_line  $lineend
#   set lineend [string tolower $lineend]
   set firstappearance [string first "source" $lineend]
   
   # CHECKS IF WORD "SOURCE" APPEARS IN LINE
   if {$firstappearance >= 0} {
      for {set k 0} {$k < $firstappearance} {incr k} {
         set carrot [string index $lineend $k]
         if {![string match " " $carrot]} {set issourcer 0; break}
      }
      set temporary [split $lineend "\."]
      if {[llength $temporary] != 2} {set issourcer 0}

      set k [expr $k+6]
      set dot [string first "\." $lineend]
      set ending [expr [string length $lineend] - 1]      

      # CHECKS IF EXTENSION IS TCL
      for {set i [expr $dot+1]} {$i < $ending} {incr i} {
         if { [expr $i == [expr $dot+1]] && \
              ![string match [string index $lineend $i] "t"]} {set issourcer 0;   break}
         if { [expr $i == [expr $dot+2]] && \
              ![string match [string index $lineend $i] "c"]} {set issourcer 0;   break}
         if { [expr $i == [expr $dot+3]] && \
              ![string match [string index $lineend $i] "l"]} {set issourcer 0;   break}
         if {![expr $i == [expr $dot+4]]} {break}
      }
   } else {set issourcer 0}

   switch $reopt {
      0 {
          return $issourcer
        }
      1 {
          if {$issourcer} {
             while {[string match [string index $lineend $k] " "]} {incr k}
             set almost_there [string trimleft [string range $final_line $k [expr $dot - 1]] /"]
             set almost_there [split $almost_there \\/]
             return [lindex $almost_there [expr [llength $almost_there]-1 ]]
          }
        }
      2 {
          if {$issourcer} {
             while {[string match [string index $lineend $k] " "]} {incr k}
             set almost_there [string trimleft [string range $final_line $k [expr $dot - 1]] /"]
             set almost_there [split $almost_there \\/]             

             set dir_ext ""

             for {set i 0} {$i < [expr [llength $almost_there]-1]} {incr i} {
                  append dir_ext [lindex $almost_there $i] "/"
             }

             return $dir_ext
          }
        }
      3 {
          if {$issourcer} {
             while {[string match [string index $lineend $k] " "]} {incr k}
             return [string range $final_line $k [expr $dot + 3]]
          }
        }
   }
}
